@@ -25,7 +25,7 @@
 #include <stdint.h>
 #include "avcodec.h"
 
-typedef struct {
+typedef struct CIDEntry {
     int cid;
     unsigned int width, height;
     int interlaced;
@@ -40,12 +40,13 @@ typedef struct {
     const uint8_t *ac_run_flag, *ac_index_flag;
     const uint16_t *run_codes;
     const uint8_t *run_bits, *run;
-    int bit_rates[5]; ///< Helpher to choose variants, rounded to nearest 5Mb/s
+    int bit_rates[5]; ///< Helper to choose variants, rounded to nearest 5Mb/s
 } CIDEntry;
 
 extern const CIDEntry ff_dnxhd_cid_table[];
 
 int ff_dnxhd_get_cid_table(int cid);
 int ff_dnxhd_find_cid(AVCodecContext *avctx, int bit_depth);
+void ff_dnxhd_list_cid(AVCodecContext *avctx);
 
 #endif /* AVCODEC_DNXHDDATA_H */
