@@ -33,8 +33,7 @@
  * @param depth     bit depth
  * @param gb        input bytestream context
  */
-int ff_msrle_decode(AVCodecContext *avctx, AVPicture *pic,
+int ff_msrle_decode(AVCodecContext *avctx, AVFrame *pic,
                     int depth, GetByteContext *gb);
 
 #endif /* AVCODEC_MSRLEDEC_H */
-
