@@ -1,6 +1,6 @@
 /*
  * Quicktime Animation (RLE) Video Decoder
- * Copyright (C) 2004 the ffmpeg project
+ * Copyright (C) 2004 The FFmpeg project
  *
  * This file is part of Libav.
  *
@@ -35,48 +35,41 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "libavutil/intreadwrite.h"
 #include "avcodec.h"
+#include "bytestream.h"
+#include "internal.h"
 
 typedef struct QtrleContext {
-
     AVCodecContext *avctx;
-    AVFrame frame;
-
-    const unsigned char *buf;
-    int size;
+    AVFrame *frame;
 
+    GetByteContext g;
     uint32_t pal[256];
 } QtrleContext;
 
-#define CHECK_STREAM_PTR(n) \
-  if ((stream_ptr + n) > s->size) { \
-    av_log (s->avctx, AV_LOG_INFO, "Problem: stream_ptr out of bounds (%d >= %d)\n", \
-      stream_ptr + n, s->size); \
-    return; \
-  }
+#define CHECK_PIXEL_PTR(n)                                                            \
+    if ((pixel_ptr + n > pixel_limit) || (pixel_ptr + n < 0)) {                       \
+        av_log (s->avctx, AV_LOG_ERROR, "Problem: pixel_ptr = %d, pixel_limit = %d\n",\
+                pixel_ptr + n, pixel_limit);                                          \
+        return;                                                                       \
+    }                                                                                 \
 
-#define CHECK_PIXEL_PTR(n) \
-  if ((pixel_ptr + n > pixel_limit) || (pixel_ptr + n < 0)) { \
-    av_log (s->avctx, AV_LOG_INFO, "Problem: pixel_ptr = %d, pixel_limit = %d\n", \
-      pixel_ptr + n, pixel_limit); \
-    return; \
-  } \
-
-static void qtrle_decode_1bpp(QtrleContext *s, int stream_ptr, int row_ptr, int lines_to_change)
+static void qtrle_decode_1bpp(QtrleContext *s, int row_ptr, int lines_to_change)
 {
     int rle_code;
-    int pixel_ptr = 0;
-    int row_inc = s->frame.linesize[0];
+    int pixel_ptr;
+    int row_inc = s->frame->linesize[0];
     unsigned char pi0, pi1;  /* 2 8-pixel values */
-    unsigned char *rgb = s->frame.data[0];
-    int pixel_limit = s->frame.linesize[0] * s->avctx->height;
+    unsigned char *rgb = s->frame->data[0];
+    int pixel_limit = s->frame->linesize[0] * s->avctx->height;
     int skip;
 
+    row_ptr  -= row_inc;
+    pixel_ptr = row_ptr;
+    lines_to_change++;
     while (lines_to_change) {
-        CHECK_STREAM_PTR(2);
-        skip = s->buf[stream_ptr++];
-        rle_code = (signed char)s->buf[stream_ptr++];
+        skip     =              bytestream2_get_byte(&s->g);
+        rle_code = (signed char)bytestream2_get_byte(&s->g);
         if (rle_code == 0)
             break;
         if(skip & 0x80) {
@@ -92,9 +85,9 @@ static void qtrle_decode_1bpp(QtrleContext *s, int stream_ptr, int row_ptr, int
             rle_code = -rle_code;
             /* get the next 2 bytes from the stream, treat them as groups
              * of 8 pixels, and output them rle_code times */
-            CHECK_STREAM_PTR(2);
-            pi0 = s->buf[stream_ptr++];
-            pi1 = s->buf[stream_ptr++];
+
+            pi0 = bytestream2_get_byte(&s->g);
+            pi1 = bytestream2_get_byte(&s->g);
             CHECK_PIXEL_PTR(rle_code * 2);
 
             while (rle_code--) {
@@ -104,45 +97,42 @@ static void qtrle_decode_1bpp(QtrleContext *s, int stream_ptr, int row_ptr, int
         } else {
             /* copy the same pixel directly to output 2 times */
             rle_code *= 2;
-            CHECK_STREAM_PTR(rle_code);
             CHECK_PIXEL_PTR(rle_code);
 
             while (rle_code--)
-                rgb[pixel_ptr++] = s->buf[stream_ptr++];
+                rgb[pixel_ptr++] = bytestream2_get_byte(&s->g);
         }
     }
 }
 
-static inline void qtrle_decode_2n4bpp(QtrleContext *s, int stream_ptr,
-                             int row_ptr, int lines_to_change, int bpp)
+static inline void qtrle_decode_2n4bpp(QtrleContext *s, int row_ptr,
+                                       int lines_to_change, int bpp)
 {
     int rle_code, i;
     int pixel_ptr;
-    int row_inc = s->frame.linesize[0];
+    int row_inc = s->frame->linesize[0];
     unsigned char pi[16];  /* 16 palette indices */
-    unsigned char *rgb = s->frame.data[0];
-    int pixel_limit = s->frame.linesize[0] * s->avctx->height;
+    unsigned char *rgb = s->frame->data[0];
+    int pixel_limit = s->frame->linesize[0] * s->avctx->height;
     int num_pixels = (bpp == 4) ? 8 : 16;
 
     while (lines_to_change--) {
-        CHECK_STREAM_PTR(2);
-        pixel_ptr = row_ptr + (num_pixels * (s->buf[stream_ptr++] - 1));
+        pixel_ptr = row_ptr + (num_pixels * (bytestream2_get_byte(&s->g) - 1));
+        CHECK_PIXEL_PTR(0);
 
-        while ((rle_code = (signed char)s->buf[stream_ptr++]) != -1) {
+        while ((rle_code = (signed char)bytestream2_get_byte(&s->g)) != -1) {
             if (rle_code == 0) {
                 /* there's another skip code in the stream */
-                CHECK_STREAM_PTR(1);
-                pixel_ptr += (num_pixels * (s->buf[stream_ptr++] - 1));
+                pixel_ptr += (num_pixels * (bytestream2_get_byte(&s->g) - 1));
                 CHECK_PIXEL_PTR(0);  /* make sure pixel_ptr is positive */
             } else if (rle_code < 0) {
                 /* decode the run length code */
                 rle_code = -rle_code;
                 /* get the next 4 bytes from the stream, treat them as palette
                  * indexes, and output them rle_code times */
-                CHECK_STREAM_PTR(4);
                 for (i = num_pixels-1; i >= 0; i--) {
-                    pi[num_pixels-1-i] = (s->buf[stream_ptr] >> ((i*bpp) & 0x07)) & ((1<<bpp)-1);
-                    stream_ptr+= ((i & ((num_pixels>>2)-1)) == 0);
+                    pi[num_pixels-1-i] = (bytestream2_peek_byte(&s->g) >> ((i*bpp) & 0x07)) & ((1<<bpp)-1);
+                    bytestream2_skip(&s->g, ((i & ((num_pixels>>2)-1)) == 0));
                 }
                 CHECK_PIXEL_PTR(rle_code * num_pixels);
                 while (rle_code--) {
@@ -152,17 +142,18 @@ static inline void qtrle_decode_2n4bpp(QtrleContext *s, int stream_ptr,
             } else {
                 /* copy the same pixel directly to output 4 times */
                 rle_code *= 4;
-                CHECK_STREAM_PTR(rle_code);
                 CHECK_PIXEL_PTR(rle_code*(num_pixels>>2));
                 while (rle_code--) {
                     if(bpp == 4) {
-                        rgb[pixel_ptr++] = ((s->buf[stream_ptr]) >> 4) & 0x0f;
-                        rgb[pixel_ptr++] = (s->buf[stream_ptr++]) & 0x0f;
+                        int x = bytestream2_get_byte(&s->g);
+                        rgb[pixel_ptr++] = (x >> 4) & 0x0f;
+                        rgb[pixel_ptr++] =  x       & 0x0f;
                     } else {
-                        rgb[pixel_ptr++] = ((s->buf[stream_ptr]) >> 6) & 0x03;
-                        rgb[pixel_ptr++] = ((s->buf[stream_ptr]) >> 4) & 0x03;
-                        rgb[pixel_ptr++] = ((s->buf[stream_ptr]) >> 2) & 0x03;
-                        rgb[pixel_ptr++] = (s->buf[stream_ptr++]) & 0x03;
+                        int x = bytestream2_get_byte(&s->g);
+                        rgb[pixel_ptr++] = (x >> 6) & 0x03;
+                        rgb[pixel_ptr++] = (x >> 4) & 0x03;
+                        rgb[pixel_ptr++] = (x >> 2) & 0x03;
+                        rgb[pixel_ptr++] =  x       & 0x03;
                     }
                 }
             }
@@ -171,35 +162,33 @@ static inline void qtrle_decode_2n4bpp(QtrleContext *s, int stream_ptr,
     }
 }
 
-static void qtrle_decode_8bpp(QtrleContext *s, int stream_ptr, int row_ptr, int lines_to_change)
+static void qtrle_decode_8bpp(QtrleContext *s, int row_ptr, int lines_to_change)
 {
     int rle_code;
     int pixel_ptr;
-    int row_inc = s->frame.linesize[0];
+    int row_inc = s->frame->linesize[0];
     unsigned char pi1, pi2, pi3, pi4;  /* 4 palette indexes */
-    unsigned char *rgb = s->frame.data[0];
-    int pixel_limit = s->frame.linesize[0] * s->avctx->height;
+    unsigned char *rgb = s->frame->data[0];
+    int pixel_limit = s->frame->linesize[0] * s->avctx->height;
 
     while (lines_to_change--) {
-        CHECK_STREAM_PTR(2);
-        pixel_ptr = row_ptr + (4 * (s->buf[stream_ptr++] - 1));
+        pixel_ptr = row_ptr + (4 * (bytestream2_get_byte(&s->g) - 1));
+        CHECK_PIXEL_PTR(0);
 
-        while ((rle_code = (signed char)s->buf[stream_ptr++]) != -1) {
+        while ((rle_code = (signed char)bytestream2_get_byte(&s->g)) != -1) {
             if (rle_code == 0) {
                 /* there's another skip code in the stream */
-                CHECK_STREAM_PTR(1);
-                pixel_ptr += (4 * (s->buf[stream_ptr++] - 1));
+                pixel_ptr += (4 * (bytestream2_get_byte(&s->g) - 1));
                 CHECK_PIXEL_PTR(0);  /* make sure pixel_ptr is positive */
             } else if (rle_code < 0) {
                 /* decode the run length code */
                 rle_code = -rle_code;
                 /* get the next 4 bytes from the stream, treat them as palette
                  * indexes, and output them rle_code times */
-                CHECK_STREAM_PTR(4);
-                pi1 = s->buf[stream_ptr++];
-                pi2 = s->buf[stream_ptr++];
-                pi3 = s->buf[stream_ptr++];
-                pi4 = s->buf[stream_ptr++];
+                pi1 = bytestream2_get_byte(&s->g);
+                pi2 = bytestream2_get_byte(&s->g);
+                pi3 = bytestream2_get_byte(&s->g);
+                pi4 = bytestream2_get_byte(&s->g);
 
                 CHECK_PIXEL_PTR(rle_code * 4);
 
@@ -212,11 +201,10 @@ static void qtrle_decode_8bpp(QtrleContext *s, int stream_ptr, int row_ptr, int
             } else {
                 /* copy the same pixel directly to output 4 times */
                 rle_code *= 4;
-                CHECK_STREAM_PTR(rle_code);
                 CHECK_PIXEL_PTR(rle_code);
 
                 while (rle_code--) {
-                    rgb[pixel_ptr++] = s->buf[stream_ptr++];
+                    rgb[pixel_ptr++] = bytestream2_get_byte(&s->g);
                 }
             }
         }
@@ -224,31 +212,28 @@ static void qtrle_decode_8bpp(QtrleContext *s, int stream_ptr, int row_ptr, int
     }
 }
 
-static void qtrle_decode_16bpp(QtrleContext *s, int stream_ptr, int row_ptr, int lines_to_change)
+static void qtrle_decode_16bpp(QtrleContext *s, int row_ptr, int lines_to_change)
 {
     int rle_code;
     int pixel_ptr;
-    int row_inc = s->frame.linesize[0];
+    int row_inc = s->frame->linesize[0];
     unsigned short rgb16;
-    unsigned char *rgb = s->frame.data[0];
-    int pixel_limit = s->frame.linesize[0] * s->avctx->height;
+    unsigned char *rgb = s->frame->data[0];
+    int pixel_limit = s->frame->linesize[0] * s->avctx->height;
 
     while (lines_to_change--) {
-        CHECK_STREAM_PTR(2);
-        pixel_ptr = row_ptr + (s->buf[stream_ptr++] - 1) * 2;
+        pixel_ptr = row_ptr + (bytestream2_get_byte(&s->g) - 1) * 2;
+        CHECK_PIXEL_PTR(0);
 
-        while ((rle_code = (signed char)s->buf[stream_ptr++]) != -1) {
+        while ((rle_code = (signed char)bytestream2_get_byte(&s->g)) != -1) {
             if (rle_code == 0) {
                 /* there's another skip code in the stream */
-                CHECK_STREAM_PTR(1);
-                pixel_ptr += (s->buf[stream_ptr++] - 1) * 2;
+                pixel_ptr += (bytestream2_get_byte(&s->g) - 1) * 2;
                 CHECK_PIXEL_PTR(0);  /* make sure pixel_ptr is positive */
             } else if (rle_code < 0) {
                 /* decode the run length code */
                 rle_code = -rle_code;
-                CHECK_STREAM_PTR(2);
-                rgb16 = AV_RB16(&s->buf[stream_ptr]);
-                stream_ptr += 2;
+                rgb16 = bytestream2_get_be16(&s->g);
 
                 CHECK_PIXEL_PTR(rle_code * 2);
 
@@ -257,13 +242,11 @@ static void qtrle_decode_16bpp(QtrleContext *s, int stream_ptr, int row_ptr, int
                     pixel_ptr += 2;
                 }
             } else {
-                CHECK_STREAM_PTR(rle_code * 2);
                 CHECK_PIXEL_PTR(rle_code * 2);
 
                 /* copy pixels directly to output */
                 while (rle_code--) {
-                    rgb16 = AV_RB16(&s->buf[stream_ptr]);
-                    stream_ptr += 2;
+                    rgb16 = bytestream2_get_be16(&s->g);
                     *(unsigned short *)(&rgb[pixel_ptr]) = rgb16;
                     pixel_ptr += 2;
                 }
@@ -273,32 +256,30 @@ static void qtrle_decode_16bpp(QtrleContext *s, int stream_ptr, int row_ptr, int
     }
 }
 
-static void qtrle_decode_24bpp(QtrleContext *s, int stream_ptr, int row_ptr, int lines_to_change)
+static void qtrle_decode_24bpp(QtrleContext *s, int row_ptr, int lines_to_change)
 {
     int rle_code;
     int pixel_ptr;
-    int row_inc = s->frame.linesize[0];
+    int row_inc = s->frame->linesize[0];
     unsigned char r, g, b;
-    unsigned char *rgb = s->frame.data[0];
-    int pixel_limit = s->frame.linesize[0] * s->avctx->height;
+    unsigned char *rgb = s->frame->data[0];
+    int pixel_limit = s->frame->linesize[0] * s->avctx->height;
 
     while (lines_to_change--) {
-        CHECK_STREAM_PTR(2);
-        pixel_ptr = row_ptr + (s->buf[stream_ptr++] - 1) * 3;
+        pixel_ptr = row_ptr + (bytestream2_get_byte(&s->g) - 1) * 3;
+        CHECK_PIXEL_PTR(0);
 
-        while ((rle_code = (signed char)s->buf[stream_ptr++]) != -1) {
+        while ((rle_code = (signed char)bytestream2_get_byte(&s->g)) != -1) {
             if (rle_code == 0) {
                 /* there's another skip code in the stream */
-                CHECK_STREAM_PTR(1);
-                pixel_ptr += (s->buf[stream_ptr++] - 1) * 3;
+                pixel_ptr += (bytestream2_get_byte(&s->g) - 1) * 3;
                 CHECK_PIXEL_PTR(0);  /* make sure pixel_ptr is positive */
             } else if (rle_code < 0) {
                 /* decode the run length code */
                 rle_code = -rle_code;
-                CHECK_STREAM_PTR(3);
-                r = s->buf[stream_ptr++];
-                g = s->buf[stream_ptr++];
-                b = s->buf[stream_ptr++];
+                r = bytestream2_get_byte(&s->g);
+                g = bytestream2_get_byte(&s->g);
+                b = bytestream2_get_byte(&s->g);
 
                 CHECK_PIXEL_PTR(rle_code * 3);
 
@@ -308,14 +289,13 @@ static void qtrle_decode_24bpp(QtrleContext *s, int stream_ptr, int row_ptr, int
                     rgb[pixel_ptr++] = b;
                 }
             } else {
-                CHECK_STREAM_PTR(rle_code * 3);
                 CHECK_PIXEL_PTR(rle_code * 3);
 
                 /* copy pixels directly to output */
                 while (rle_code--) {
-                    rgb[pixel_ptr++] = s->buf[stream_ptr++];
-                    rgb[pixel_ptr++] = s->buf[stream_ptr++];
-                    rgb[pixel_ptr++] = s->buf[stream_ptr++];
+                    rgb[pixel_ptr++] = bytestream2_get_byte(&s->g);
+                    rgb[pixel_ptr++] = bytestream2_get_byte(&s->g);
+                    rgb[pixel_ptr++] = bytestream2_get_byte(&s->g);
                 }
             }
         }
@@ -323,31 +303,28 @@ static void qtrle_decode_24bpp(QtrleContext *s, int stream_ptr, int row_ptr, int
     }
 }
 
-static void qtrle_decode_32bpp(QtrleContext *s, int stream_ptr, int row_ptr, int lines_to_change)
+static void qtrle_decode_32bpp(QtrleContext *s, int row_ptr, int lines_to_change)
 {
     int rle_code;
     int pixel_ptr;
-    int row_inc = s->frame.linesize[0];
+    int row_inc = s->frame->linesize[0];
     unsigned int argb;
-    unsigned char *rgb = s->frame.data[0];
-    int pixel_limit = s->frame.linesize[0] * s->avctx->height;
+    unsigned char *rgb = s->frame->data[0];
+    int pixel_limit = s->frame->linesize[0] * s->avctx->height;
 
     while (lines_to_change--) {
-        CHECK_STREAM_PTR(2);
-        pixel_ptr = row_ptr + (s->buf[stream_ptr++] - 1) * 4;
+        pixel_ptr = row_ptr + (bytestream2_get_byte(&s->g) - 1) * 4;
+        CHECK_PIXEL_PTR(0);
 
-        while ((rle_code = (signed char)s->buf[stream_ptr++]) != -1) {
+        while ((rle_code = (signed char)bytestream2_get_byte(&s->g)) != -1) {
             if (rle_code == 0) {
                 /* there's another skip code in the stream */
-                CHECK_STREAM_PTR(1);
-                pixel_ptr += (s->buf[stream_ptr++] - 1) * 4;
+                pixel_ptr += (bytestream2_get_byte(&s->g) - 1) * 4;
                 CHECK_PIXEL_PTR(0);  /* make sure pixel_ptr is positive */
             } else if (rle_code < 0) {
                 /* decode the run length code */
                 rle_code = -rle_code;
-                CHECK_STREAM_PTR(4);
-                argb = AV_RB32(s->buf + stream_ptr);
-                stream_ptr += 4;
+                argb = bytestream2_get_be32(&s->g);
 
                 CHECK_PIXEL_PTR(rle_code * 4);
 
@@ -356,14 +333,12 @@ static void qtrle_decode_32bpp(QtrleContext *s, int stream_ptr, int row_ptr, int
                     pixel_ptr += 4;
                 }
             } else {
-                CHECK_STREAM_PTR(rle_code * 4);
                 CHECK_PIXEL_PTR(rle_code * 4);
 
                 /* copy pixels directly to output */
                 while (rle_code--) {
-                    argb = AV_RB32(s->buf + stream_ptr);
+                    argb = bytestream2_get_be32(&s->g);
                     AV_WN32A(rgb + pixel_ptr, argb);
-                    stream_ptr += 4;
                     pixel_ptr  += 4;
                 }
             }
@@ -380,7 +355,7 @@ static av_cold int qtrle_decode_init(AVCodecContext *avctx)
     switch (avctx->bits_per_coded_sample) {
     case 1:
     case 33:
-        avctx->pix_fmt = PIX_FMT_MONOWHITE;
+        avctx->pix_fmt = AV_PIX_FMT_MONOWHITE;
         break;
 
     case 2:
@@ -389,113 +364,108 @@ static av_cold int qtrle_decode_init(AVCodecContext *avctx)
     case 34:
     case 36:
     case 40:
-        avctx->pix_fmt = PIX_FMT_PAL8;
+        avctx->pix_fmt = AV_PIX_FMT_PAL8;
         break;
 
     case 16:
-        avctx->pix_fmt = PIX_FMT_RGB555;
+        avctx->pix_fmt = AV_PIX_FMT_RGB555;
         break;
 
     case 24:
-        avctx->pix_fmt = PIX_FMT_RGB24;
+        avctx->pix_fmt = AV_PIX_FMT_RGB24;
         break;
 
     case 32:
-        avctx->pix_fmt = PIX_FMT_RGB32;
+        avctx->pix_fmt = AV_PIX_FMT_RGB32;
         break;
 
     default:
         av_log (avctx, AV_LOG_ERROR, "Unsupported colorspace: %d bits/sample?\n",
             avctx->bits_per_coded_sample);
-        break;
+        return AVERROR_INVALIDDATA;
     }
 
-    s->frame.data[0] = NULL;
+    s->frame = av_frame_alloc();
+    if (!s->frame)
+        return AVERROR(ENOMEM);
 
     return 0;
 }
 
 static int qtrle_decode_frame(AVCodecContext *avctx,
-                              void *data, int *data_size,
+                              void *data, int *got_frame,
                               AVPacket *avpkt)
 {
-    const uint8_t *buf = avpkt->data;
-    int buf_size = avpkt->size;
     QtrleContext *s = avctx->priv_data;
     int header, start_line;
-    int stream_ptr, height, row_ptr;
+    int height, row_ptr;
     int has_palette = 0;
+    int ret;
 
-    s->buf = buf;
-    s->size = buf_size;
-
-    s->frame.reference = 1;
-    s->frame.buffer_hints = FF_BUFFER_HINTS_VALID | FF_BUFFER_HINTS_PRESERVE |
-                            FF_BUFFER_HINTS_REUSABLE | FF_BUFFER_HINTS_READABLE;
-    if (avctx->reget_buffer(avctx, &s->frame)) {
+    bytestream2_init(&s->g, avpkt->data, avpkt->size);
+    if ((ret = ff_reget_buffer(avctx, s->frame)) < 0) {
         av_log (s->avctx, AV_LOG_ERROR, "reget_buffer() failed\n");
-        return -1;
+        return ret;
     }
 
     /* check if this frame is even supposed to change */
-    if (s->size < 8)
+    if (avpkt->size < 8)
         goto done;
 
     /* start after the chunk size */
-    stream_ptr = 4;
+    bytestream2_seek(&s->g, 4, SEEK_SET);
 
     /* fetch the header */
-    header = AV_RB16(&s->buf[stream_ptr]);
-    stream_ptr += 2;
+    header = bytestream2_get_be16(&s->g);
 
     /* if a header is present, fetch additional decoding parameters */
     if (header & 0x0008) {
-        if(s->size < 14)
+        if (avpkt->size < 14)
             goto done;
-        start_line = AV_RB16(&s->buf[stream_ptr]);
-        stream_ptr += 4;
-        height = AV_RB16(&s->buf[stream_ptr]);
-        stream_ptr += 4;
+        start_line = bytestream2_get_be16(&s->g);
+        bytestream2_skip(&s->g, 2);
+        height     = bytestream2_get_be16(&s->g);
+        bytestream2_skip(&s->g, 2);
     } else {
         start_line = 0;
-        height = s->avctx->height;
+        height     = s->avctx->height;
     }
-    row_ptr = s->frame.linesize[0] * start_line;
+    row_ptr = s->frame->linesize[0] * start_line;
 
     switch (avctx->bits_per_coded_sample) {
     case 1:
     case 33:
-        qtrle_decode_1bpp(s, stream_ptr, row_ptr, height);
+        qtrle_decode_1bpp(s, row_ptr, height);
         break;
 
     case 2:
     case 34:
-        qtrle_decode_2n4bpp(s, stream_ptr, row_ptr, height, 2);
+        qtrle_decode_2n4bpp(s, row_ptr, height, 2);
         has_palette = 1;
         break;
 
     case 4:
     case 36:
-        qtrle_decode_2n4bpp(s, stream_ptr, row_ptr, height, 4);
+        qtrle_decode_2n4bpp(s, row_ptr, height, 4);
         has_palette = 1;
         break;
 
     case 8:
     case 40:
-        qtrle_decode_8bpp(s, stream_ptr, row_ptr, height);
+        qtrle_decode_8bpp(s, row_ptr, height);
         has_palette = 1;
         break;
 
     case 16:
-        qtrle_decode_16bpp(s, stream_ptr, row_ptr, height);
+        qtrle_decode_16bpp(s, row_ptr, height);
         break;
 
     case 24:
-        qtrle_decode_24bpp(s, stream_ptr, row_ptr, height);
+        qtrle_decode_24bpp(s, row_ptr, height);
         break;
 
     case 32:
-        qtrle_decode_32bpp(s, stream_ptr, row_ptr, height);
+        qtrle_decode_32bpp(s, row_ptr, height);
         break;
 
     default:
@@ -508,41 +478,40 @@ static int qtrle_decode_frame(AVCodecContext *avctx,
         const uint8_t *pal = av_packet_get_side_data(avpkt, AV_PKT_DATA_PALETTE, NULL);
 
         if (pal) {
-            s->frame.palette_has_changed = 1;
+            s->frame->palette_has_changed = 1;
             memcpy(s->pal, pal, AVPALETTE_SIZE);
         }
 
         /* make the palette available on the way out */
-        memcpy(s->frame.data[1], s->pal, AVPALETTE_SIZE);
+        memcpy(s->frame->data[1], s->pal, AVPALETTE_SIZE);
     }
 
 done:
-    *data_size = sizeof(AVFrame);
-    *(AVFrame*)data = s->frame;
+    if ((ret = av_frame_ref(data, s->frame)) < 0)
+        return ret;
+    *got_frame      = 1;
 
     /* always report that the buffer was completely consumed */
-    return buf_size;
+    return avpkt->size;
 }
 
 static av_cold int qtrle_decode_end(AVCodecContext *avctx)
 {
     QtrleContext *s = avctx->priv_data;
 
-    if (s->frame.data[0])
-        avctx->release_buffer(avctx, &s->frame);
+    av_frame_free(&s->frame);
 
     return 0;
 }
 
 AVCodec ff_qtrle_decoder = {
     .name           = "qtrle",
+    .long_name      = NULL_IF_CONFIG_SMALL("QuickTime Animation (RLE) video"),
     .type           = AVMEDIA_TYPE_VIDEO,
-    .id             = CODEC_ID_QTRLE,
+    .id             = AV_CODEC_ID_QTRLE,
     .priv_data_size = sizeof(QtrleContext),
     .init           = qtrle_decode_init,
     .close          = qtrle_decode_end,
     .decode         = qtrle_decode_frame,
-    .capabilities   = CODEC_CAP_DR1,
-    .long_name = NULL_IF_CONFIG_SMALL("QuickTime Animation (RLE) video"),
+    .capabilities   = AV_CODEC_CAP_DR1,
 };
-
